@@ -1,10 +1,11 @@
 package DBM::Deep::Array;
 
-use 5.008_000;
+use 5.006_000;
 
 use strict;
-use warnings FATAL => 'all';
-no warnings 'recursion';
+use warnings;
+
+our $VERSION = q(1.0001);
 
 # This is to allow DBM::Deep::Array to handle negative indices on
 # its own. Otherwise, Perl would intercept the call to negative
@@ -16,18 +17,19 @@ use base 'DBM::Deep';
 use Scalar::Util ();
 
 sub _get_self {
-    # We used to have
-    #  eval { local $SIG{'__DIE__'}; tied( @{$_[0]} ) } || $_[0]
-    # but this does not always work during global destruction (DBM::Deep’s
-    # destructor calls this method), but will return $_[0] even when $_[0]
-    # is tied, if it’s tied to undef. In those cases it’s better to return
-    # undef, so the destructor can tell not to do anything,  and,  if any-
-    # thing else calls us, it will fail with a more helpful error message.
-
-    Scalar::Util::reftype $_[0] eq 'ARRAY' ? tied @{$_[0]} : $_[0];
+    eval { local $SIG{'__DIE__'}; tied( @{$_[0]} ) } || $_[0]
 }
 
-sub _repr { [] }
+sub _repr { shift;[ @_ ] }
+
+sub _import {
+    my $self = shift;
+    my ($struct) = @_;
+
+    $self->push( @$struct );
+
+    return 1;
+}
 
 sub TIEARRAY {
     my $class = shift;
@@ -42,10 +44,9 @@ sub FETCH {
     my $self = shift->_get_self;
     my ($key) = @_;
 
-    $self->lock_shared;
+    $self->lock( $self->LOCK_SH );
 
     if ( !defined $key ) {
-        $self->unlock;
         DBM::Deep->_throw_error( "Cannot use an undefined array index." );
     }
     elsif ( $key =~ /^-?\d+$/ ) {
@@ -73,12 +74,11 @@ sub STORE {
     my $self = shift->_get_self;
     my ($key, $value) = @_;
 
-    $self->lock_exclusive;
+    $self->lock( $self->LOCK_EX );
 
     my $size;
     my $idx_is_numeric;
     if ( !defined $key ) {
-        $self->unlock;
         DBM::Deep->_throw_error( "Cannot use an undefined array index." );
     }
     elsif ( $key =~ /^-?\d+$/ ) {
@@ -114,10 +114,9 @@ sub EXISTS {
     my $self = shift->_get_self;
     my ($key) = @_;
 
-    $self->lock_shared;
+    $self->lock( $self->LOCK_SH );
 
     if ( !defined $key ) {
-        $self->unlock;
         DBM::Deep->_throw_error( "Cannot use an undefined array index." );
     }
     elsif ( $key =~ /^-?\d+$/ ) {
@@ -144,13 +143,11 @@ sub EXISTS {
 sub DELETE {
     my $self = shift->_get_self;
     my ($key) = @_;
-    warn "ARRAY::DELETE($self,$key)\n" if DBM::Deep::DEBUG;
 
-    $self->lock_exclusive;
+    $self->lock( $self->LOCK_EX );
 
     my $size = $self->FETCHSIZE;
     if ( !defined $key ) {
-        $self->unlock;
         DBM::Deep->_throw_error( "Cannot use an undefined array index." );
     }
     elsif ( $key =~ /^-?\d+$/ ) {
@@ -178,20 +175,20 @@ sub DELETE {
     return $rv;
 }
 
-# Now that we have a real Reference sector, we should store arrayzize there.
-# However, arraysize needs to be transactionally-aware, so a simple location to
-# store it isn't going to work.
+# Now that we have a real Reference sector, we should store arrayzize there. However,
+# arraysize needs to be transactionally-aware, so a simple location to store it isn't
+# going to work.
 sub FETCHSIZE {
     my $self = shift->_get_self;
 
-    $self->lock_shared;
+    $self->lock( $self->LOCK_SH );
 
-    my $SAVE_FILTER = $self->_engine->storage->{filter_fetch_value};
-    $self->_engine->storage->{filter_fetch_value} = undef;
+    my $SAVE_FILTER = $self->_storage->{filter_fetch_value};
+    $self->_storage->{filter_fetch_value} = undef;
 
     my $size = $self->FETCH('length') || 0;
 
-    $self->_engine->storage->{filter_fetch_value} = $SAVE_FILTER;
+    $self->_storage->{filter_fetch_value} = $SAVE_FILTER;
 
     $self->unlock;
 
@@ -202,14 +199,14 @@ sub STORESIZE {
     my $self = shift->_get_self;
     my ($new_length) = @_;
 
-    $self->lock_exclusive;
+    $self->lock( $self->LOCK_EX );
 
-    my $SAVE_FILTER = $self->_engine->storage->{filter_store_value};
-    $self->_engine->storage->{filter_store_value} = undef;
+    my $SAVE_FILTER = $self->_storage->{filter_store_value};
+    $self->_storage->{filter_store_value} = undef;
 
     my $result = $self->STORE('length', $new_length, 'length');
 
-    $self->_engine->storage->{filter_store_value} = $SAVE_FILTER;
+    $self->_storage->{filter_store_value} = $SAVE_FILTER;
 
     $self->unlock;
 
@@ -219,7 +216,7 @@ sub STORESIZE {
 sub POP {
     my $self = shift->_get_self;
 
-    $self->lock_exclusive;
+    $self->lock( $self->LOCK_EX );
 
     my $length = $self->FETCHSIZE();
 
@@ -240,7 +237,7 @@ sub POP {
 sub PUSH {
     my $self = shift->_get_self;
 
-    $self->lock_exclusive;
+    $self->lock( $self->LOCK_EX );
 
     my $length = $self->FETCHSIZE();
 
@@ -254,59 +251,44 @@ sub PUSH {
     return $length;
 }
 
-# XXX This really needs to be something more direct within the file, not a
-# fetch and re-store. -RobK, 2007-09-20
-sub _move_value {
-    my $self = shift;
-    my ($old_key, $new_key) = @_;
-
-    return $self->_engine->make_reference( $self, $old_key, $new_key );
-}
-
 sub SHIFT {
     my $self = shift->_get_self;
-    warn "SHIFT($self)\n" if DBM::Deep::DEBUG;
 
-    $self->lock_exclusive;
+    $self->lock( $self->LOCK_EX );
 
     my $length = $self->FETCHSIZE();
 
-    if ( !$length ) {
+    if ($length) {
+        my $content = $self->FETCH( 0 );
+
+        for (my $i = 0; $i < $length - 1; $i++) {
+            $self->STORE( $i, $self->FETCH($i + 1) );
+        }
+        $self->DELETE( $length - 1 );
+
+        $self->unlock;
+
+        return $content;
+    }
+    else {
         $self->unlock;
         return;
     }
-
-    my $content = $self->DELETE( 0 );
-
-    # Unless the deletion above has cleared the array ...
-    if ( $length > 1 ) {
-        for (my $i = 0; $i < $length - 1; $i++) {
-            $self->_move_value( $i+1, $i );
-        }
-
-        $self->DELETE( $length - 1 );
-    }
-
-    $self->unlock;
-
-    return $content;
 }
 
 sub UNSHIFT {
     my $self = shift->_get_self;
     my @new_elements = @_;
 
-    $self->lock_exclusive;
+    $self->lock( $self->LOCK_EX );
 
     my $length = $self->FETCHSIZE();
     my $new_size = scalar @new_elements;
 
     if ($length) {
         for (my $i = $length - 1; $i >= 0; $i--) {
-            $self->_move_value( $i, $i+$new_size );
+            $self->STORE( $i + $new_size, $self->FETCH($i) );
         }
-
-        $self->STORESIZE( $length + $new_size );
     }
 
     for (my $i = 0; $i < $new_size; $i++) {
@@ -321,7 +303,7 @@ sub UNSHIFT {
 sub SPLICE {
     my $self = shift->_get_self;
 
-    $self->lock_exclusive;
+    $self->lock( $self->LOCK_EX );
 
     my $length = $self->FETCHSIZE();
 
@@ -353,13 +335,12 @@ sub SPLICE {
     if ( $new_size != $splice_length ) {
         if ($new_size > $splice_length) {
             for (my $i = $length - 1; $i >= $offset + $splice_length; $i--) {
-                $self->_move_value( $i, $i + ($new_size - $splice_length) );
+                $self->STORE( $i + ($new_size - $splice_length), $self->FETCH($i) );
             }
-            $self->STORESIZE( $length + $new_size - $splice_length );
         }
         else {
             for (my $i = $offset + $splice_length; $i < $length; $i++) {
-                $self->_move_value( $i, $i + ($new_size - $splice_length) );
+                $self->STORE( $i + ($new_size - $splice_length), $self->FETCH($i) );
             }
             for (my $i = 0; $i < $splice_length - $new_size; $i++) {
                 $self->DELETE( $length - 1 );
@@ -385,9 +366,12 @@ sub SPLICE {
 
 # We don't need to populate it, yet.
 # It will be useful, though, when we split out HASH and ARRAY
-# Perl will call EXTEND() when the array is likely to grow.
-# We don't care, but include it because it gets called at times.
-sub EXTEND {}
+sub EXTEND {
+    ##
+    # Perl will call EXTEND() when the array is likely to grow.
+    # We don't care, but include it because it gets called at times.
+    ##
+}
 
 sub _copy_node {
     my $self = shift;
@@ -395,29 +379,21 @@ sub _copy_node {
 
     my $length = $self->length();
     for (my $index = 0; $index < $length; $index++) {
-        $self->_copy_value( \$db_temp->[$index], $self->get($index) );
+        my $value = $self->get($index);
+        $self->_copy_value( \$db_temp->[$index], $value );
     }
 
     return 1;
 }
 
-sub _clear {
-    my $self = shift;
-
-    my $size = $self->FETCHSIZE;
-    for my $key ( 0 .. $size - 1 ) {
-        $self->_engine->delete_key( $self, $key, $key );
-    }
-    $self->STORESIZE( 0 );
-
-    return;
-}
-
-sub length  { (shift)->FETCHSIZE(@_) }
-sub pop     { (shift)->POP(@_)       }
-sub push    { (shift)->PUSH(@_)      }
-sub unshift { (shift)->UNSHIFT(@_)   }
-sub splice  { (shift)->SPLICE(@_)    }
+##
+# Public method aliases
+##
+sub length { (shift)->FETCHSIZE(@_) }
+sub pop { (shift)->POP(@_) }
+sub push { (shift)->PUSH(@_) }
+sub unshift { (shift)->UNSHIFT(@_) }
+sub splice { (shift)->SPLICE(@_) }
 
 # This must be last otherwise we have to qualify all other calls to shift
 # as calls to CORE::shift
